/* Matrix (c) 1998 Romain GUY
   This is a PACKAGE (use import Matrix;)
   Use it for free and for the good of the third dimension !!

   This Class can be used for operations on matrices.
   When you don't specify the good dimensions for your matrices,
   the procedure returns the first matrix passed in arguments.

   powerteam@chez.com
   www.chez.com/powerteam

   Methods:

     printMat(float[][] Mat) : display contents of a matrix

     printMat(int[][] Mat) : display contents of a matrix

     toInt(float[][] Mat) : convert a float matrix to int

     abs(float[][] Mat) : returns the absolute value of a matrix

     transpose(float[][] Mat) : transpose rows to columns

     scale(float scale, float[][] Mat) : multiply a matrix by a single float

     add(float[][] Mat_A, float[][] Mat_B) : add two matrices

     multiply(float[][] Mat_A, float[][] Mat_B) : multiply a matrix by another one

     identity(int size): creates a indentity (squared) matrix

     square(float[][] Mat) : returns the squared matrix

   Your matrices must follow the following rules:
     - For the scale and the transpose methodes, size of the rows and the columns doesn't matter,
     - For the add method, both matrixes must have the same size,
     - For the multiply method, both matrices must be squared (ex: float[2][2]),
*/

package Matrix;

public class Matrix
{
  public Matrix() { }

  public static float[][] indentity(int size)
  {
    float[][] Mat_ID = new float[size][size];
    for (int i = 0; i < size; i++)
      Mat_ID[i][i] = 1;
    return Mat_ID;
  }

  public static float[][] transpose(float Mat[][])
  {
    int matLength = Mat.length;
    int matLengthSecond = Mat[1].length;
    float[][] Mat_Trans = new float[matLengthSecond][matLength];
    for (int i = 0; i < matLength; i++)
      for (int j = 0; j < matLengthSecond; j++)
        Mat_Trans[j][i] = Mat[i][j];
    return Mat_Trans;
  }

  public static float[][] scale(float scaleFactor, float Mat[][])
  {
    int matLength = Mat.length;
    int matLengthSecond = Mat[1].length;
    float[][] Mat_Scale = new float[matLength][matLengthSecond];
    for (int i = 0; i < matLength; i++)
      for (int j = 0; j < matLengthSecond; j++)
        Mat_Scale[i][j] = Mat[i][j] * scaleFactor;
    return Mat_Scale;
  }

  public static float[][] add(float Mat_A[][], float Mat_B[][])
  {
    int matLength = Mat_A.length;
    int matLengthSecond = Mat_A[1].length;
    if ((matLength != Mat_B.length) || (matLengthSecond != Mat_B[1].length)) return Mat_A;
    float[][] Mat_Add = new float[matLength][matLengthSecond];
    for (int i = 0; i < matLength; i++)
      for (int j = 0; j < matLengthSecond; j++)
        Mat_Add[i][j] = Mat_A[i][j] + Mat_B[i][j];
    return Mat_Add;
  }

  public static float[][] multiply(float Mat_A[][], float Mat_B[][])
  {
    int matLength = Mat_A.length;
    if (matLength != Mat_A[1].length) return Mat_A;
    if ((matLength != Mat_B.length) || (matLength != Mat_B[1].length)) return Mat_A;
    float[][] Mat_Multi = new float[matLength][matLength];
    for (int i = 0; i < matLength; i++)
      for (int j = 0; j < matLength; j++)
        for (int k = 0; k < matLength; k++)
          Mat_Multi[i][j] += Mat_A[i][k] * Mat_B[k][j];
    return Mat_Multi;
  }

  public static float[][] square(float Mat[][])
  {
    return multiply(Mat, Mat);
  }

  public static float[][] abs(float Mat[][])
  {
    int matLength = Mat.length;
    int matLengthSecond = Mat[1].length;
    float[][] Mat_Abs = new float[matLength][matLengthSecond];
    for (int i = 0; i < matLength; i++)
      for (int j = 0; j < matLengthSecond; j++)
        Mat_Abs[i][j] = Math.abs(Mat[i][j]);
    return Mat_Abs;
  }

  public static int[][] toInt(float Mat[][])
  {
    int matLength = Mat.length;
    int matLengthSecond = Mat[1].length;
    int[][] Mat_Int = new int[matLength][matLengthSecond];
    for (int i = 0; i < matLength; i++)
      for (int j = 0; j < matLengthSecond; j++)
        Mat_Int[i][j] = (int) Mat[i][j];
    return Mat_Int;
  }

  public static void printMat(float Mat[][])
  {
    int matLength = Mat.length;
    int matLengthSecond = Mat[1].length;
    for (int i = 0; i < matLength; i++)
    {
      System.out.print("[ ");
      for (int j = 0; j < matLengthSecond; j++)
        System.out.print(Mat[i][j] + " ");
      System.out.println("]");
    }
  }

  public static void printMat(int Mat[][])
  {
    int matLength = Mat.length;
    int matLengthSecond = Mat[1].length;
    for (int i = 0; i < matLength; i++)
    {
      System.out.print("[ ");
      for (int j = 0; j < matLengthSecond; j++)
        System.out.print(Mat[i][j] + " ");
      System.out.println("]");
    }
  }
}

// End Of Source
